@@ -19,6 +19,9 @@ use crate::core::ribosome::guest_callback::init::InitResult;
 use crate::core::ribosome::real_ribosome::RealRibosome;
 use crate::core::ribosome::ZomeCallInvocation;
 use crate::core::workflow::call_zome_workflow;
+use crate::core::workflow::countersigning_workflow::countersigning_success;
+use crate::core::workflow::countersigning_workflow::incoming_countersigning;
+use crate::core::workflow::countersigning_workflow::CountersigningWorkspace;
 use crate::core::workflow::genesis_workflow::genesis_workflow;
 use crate::core::workflow::incoming_dht_ops_workflow::incoming_dht_ops_workflow;
 use crate::core::workflow::initialize_zomes_workflow;
@@ -34,12 +37,16 @@ use hash_type::AnyDht;
 use holo_hash::*;
 use holochain_cascade::authority;
 use holochain_cascade::Cascade;
+use holochain_p2p::dht_arc::ArcInterval;
+use holochain_p2p::dht_arc::DhtArcSet;
 use holochain_serialized_bytes::SerializedBytes;
 use holochain_sqlite::prelude::*;
 use holochain_state::host_fn_workspace::HostFnWorkspace;
 use holochain_state::prelude::*;
 use holochain_types::prelude::*;
+use kitsune_p2p::event::TimeWindowMs;
 use observability::OpenSpanExt;
+use rusqlite::OptionalExtension;
 use std::hash::Hash;
 use std::hash::Hasher;
 use tokio::sync;
@@ -100,6 +107,8 @@ where
     holochain_p2p_cell: P2pCell,
     queue_triggers: QueueTriggers,
     init_mutex: tokio::sync::Mutex<()>,
+    /// Countersigning workspace that is shared across this cell.
+    countersigning_workspace: CountersigningWorkspace,
 }
 
 impl Cell {
@@ -129,6 +138,7 @@ impl Cell {
         };
 
         if has_genesis {
+            let countersigning_workspace = CountersigningWorkspace::new();
             let (queue_triggers, initial_queue_triggers) = spawn_queue_consumer_tasks(
                 env.clone(),
                 cache.clone(),
@@ -137,6 +147,7 @@ impl Cell {
                 conductor_api.clone(),
                 managed_task_add_sender,
                 managed_task_stop_broadcaster,
+                countersigning_workspace.clone(),
             )
             .await;
 
@@ -149,6 +160,7 @@ impl Cell {
                     holochain_p2p_cell,
                     queue_triggers,
                     init_mutex: Default::default(),
+                    countersigning_workspace,
                 },
                 initial_queue_triggers,
             ))
@@ -232,13 +244,13 @@ impl Cell {
             PutAgentInfoSigned { .. }
             | GetAgentInfoSigned { .. }
             | QueryAgentInfoSigned { .. }
-            | QueryAgentInfoSignedNearBasis { .. } => {
-                // PutAgentInfoSigned needs to be handled at the conductor level where the p2p
-                // store lives.
-                unreachable!()
-            }
-            PutMetricDatum { .. } | QueryMetrics { .. } => {
-                // Same goes for metrics
+            | QueryGossipAgents { .. }
+            | QueryOpHashes { .. }
+            | QueryAgentInfoSignedNearBasis { .. }
+            | QueryPeerDensity { .. }
+            | PutMetricDatum { .. }
+            | QueryMetrics { .. } => {
+                // These events are aggregated over a set of cells, so need to be handled at the conductor level.
                 unreachable!()
             }
             CallRemote {
@@ -264,16 +276,15 @@ impl Cell {
             Publish {
                 span_context,
                 respond,
-                from_agent,
                 request_validation_receipt,
-                dht_hash,
+                countersigning_session,
                 ops,
                 ..
             } => {
                 async {
                     tracing::Span::set_current_context(span_context);
                     let res = self
-                        .handle_publish(from_agent, request_validation_receipt, dht_hash, ops)
+                        .handle_publish(request_validation_receipt, countersigning_session, ops)
                         .await
                         .map_err(holochain_p2p::HolochainP2pError::other);
                     respond.respond(Ok(async move { res }.boxed().into()));
@@ -382,25 +393,8 @@ impl Cell {
                 .instrument(debug_span!("cell_handle_validation_receipt_received"))
                 .await;
             }
-            FetchOpHashesForConstraints {
-                span_context: _,
-                respond,
-                dht_arc,
-                since,
-                until,
-                ..
-            } => {
-                async {
-                    let res = self
-                        .handle_fetch_op_hashes_for_constraints(dht_arc, since, until)
-                        .await
-                        .map_err(holochain_p2p::HolochainP2pError::other);
-                    respond.respond(Ok(async move { res }.boxed().into()));
-                }
-                .instrument(debug_span!("cell_handle_fetch_op_hashes_for_constraints"))
-                .await;
-            }
-            FetchOpHashData {
+
+            FetchOpData {
                 span_context: _,
                 respond,
                 op_hashes,
@@ -408,12 +402,12 @@ impl Cell {
             } => {
                 async {
                     let res = self
-                        .handle_fetch_op_hash_data(op_hashes)
+                        .handle_fetch_op_data(op_hashes)
                         .await
                         .map_err(holochain_p2p::HolochainP2pError::other);
                     respond.respond(Ok(async move { res }.boxed().into()));
                 }
-                .instrument(debug_span!("cell_handle_fetch_op_hash_data"))
+                .instrument(debug_span!("cell_handle_fetch_op_data"))
                 .await;
             }
             SignNetworkData {
@@ -431,31 +425,75 @@ impl Cell {
                 .instrument(debug_span!("cell_handle_sign_network_data"))
                 .await;
             }
+            CountersigningAuthorityResponse {
+                respond,
+                signed_headers,
+                ..
+            } => {
+                async {
+                    let res = self
+                        .handle_countersigning_authority_response(signed_headers)
+                        .await
+                        .map_err(holochain_p2p::HolochainP2pError::other);
+                    respond.respond(Ok(async move { res }.boxed().into()));
+                }
+                .instrument(debug_span!("cell_handle_countersigning_response"))
+                .await;
+            }
         }
         Ok(())
     }
 
-    #[instrument(skip(self, request_validation_receipt, _dht_hash, ops))]
+    #[instrument(skip(self, request_validation_receipt, ops))]
     /// we are receiving a "publish" event from the network
     async fn handle_publish(
         &self,
-        from_agent: AgentPubKey,
         request_validation_receipt: bool,
-        _dht_hash: holo_hash::AnyDhtHash,
+        countersigning_session: bool,
         ops: Vec<(holo_hash::DhtOpHash, holochain_types::dht_op::DhtOp)>,
     ) -> CellResult<()> {
-        incoming_dht_ops_workflow(
-            &self.env,
-            self.queue_triggers.sys_validation.clone(),
-            ops,
-            Some(from_agent),
-            request_validation_receipt,
+        // If this is a countersigning session then
+        // send it to the countersigning workflow otherwise
+        // send it to the incoming ops workflow.
+        if countersigning_session {
+            incoming_countersigning(
+                ops,
+                &self.countersigning_workspace,
+                self.queue_triggers.countersigning.clone(),
+            )
+            .map_err(Box::new)
+            .map_err(ConductorApiError::from)
+            .map_err(Box::new)?;
+        } else {
+            incoming_dht_ops_workflow(
+                &self.env,
+                self.queue_triggers.sys_validation.clone(),
+                ops,
+                request_validation_receipt,
+            )
+            .await
+            .map_err(Box::new)
+            .map_err(ConductorApiError::from)
+            .map_err(Box::new)?;
+        }
+        Ok(())
+    }
+    #[instrument(skip(self, signed_headers))]
+    /// we are receiving a response from a countersigning authority
+    async fn handle_countersigning_authority_response(
+        &self,
+        signed_headers: Vec<SignedHeader>,
+    ) -> CellResult<()> {
+        Ok(countersigning_success(
+            self.env.clone(),
+            &self.holochain_p2p_cell,
+            self.id.agent_pubkey().clone(),
+            signed_headers,
+            self.queue_triggers.publish_dht_ops.clone(),
+            self.conductor_api.signal_broadcaster().await,
         )
         .await
-        .map_err(Box::new)
-        .map_err(ConductorApiError::from)
-        .map_err(Box::new)?;
-        Ok(())
+        .map_err(Box::new)?)
     }
 
     #[instrument(skip(self))]
@@ -594,14 +632,71 @@ impl Cell {
     }
 
     /// a remote agent is sending us a validation receipt.
-    #[tracing::instrument(skip(self))]
+    #[tracing::instrument(skip(self, receipt))]
     async fn handle_validation_receipt(&self, receipt: SerializedBytes) -> CellResult<()> {
         let receipt: SignedValidationReceipt = receipt.try_into()?;
+        tracing::debug!(from = ?receipt.receipt.validator, to = ?self.id.agent_pubkey(), hash = ?receipt.receipt.dht_op_hash);
+
+        // Get the header for this op so we can check the entry type.
+        let header: Option<SignedHeader> = fresh_reader!(self.env, |txn| {
+            let h: Option<Vec<u8>> = txn
+                .query_row(
+                    "SELECT Header.blob as header_blob
+                    FROM DhtOp 
+                    JOIN Header ON Header.hash = DhtOp.header_hash
+                    WHERE DhtOp.hash = :hash",
+                    named_params! {
+                        ":hash": receipt.receipt.dht_op_hash,
+                    },
+                    |row| row.get("header_blob"),
+                )
+                .optional()?;
+            match h {
+                Some(h) => from_blob(h),
+                None => Ok(None),
+            }
+        })?;
+
+        // If the header has an app entry type get the entry def
+        // from the conductor.
+        let required_receipt_count = match header.as_ref().and_then(|h| h.0.entry_type()) {
+            Some(EntryType::App(entry_type)) => {
+                let zome_index = u8::from(entry_type.zome_id()) as usize;
+                let dna_file = self.conductor_api.get_this_dna().await.map_err(Box::new)?;
+                let zome = dna_file.dna().zomes.get(zome_index).map(|(_, z)| z.clone());
+                match zome {
+                    Some(zome) => self
+                        .conductor_api
+                        .get_entry_def(&EntryDefBufferKey::new(zome, entry_type.id()))
+                        .await
+                        .map(|e| u8::from(e.required_validations)),
+                    None => None,
+                }
+            }
+            _ => None,
+        };
+
+        // If no required receipt count was found then fallback to the default.
+        let required_validation_count = required_receipt_count.unwrap_or(
+            crate::core::workflow::publish_dht_ops_workflow::DEFAULT_RECEIPT_BUNDLE_SIZE,
+        );
 
         self.env
             .async_commit(move |txn| {
-                // Update receipt count.
-                add_one_receipt_count(txn, &receipt.receipt.dht_op_hash)?;
+                // Get the current count for this dhtop.
+                let receipt_count: usize = txn.query_row(
+                    "SELECT COUNT(rowid) FROM ValidationReceipt WHERE op_hash = :op_hash",
+                    named_params! {
+                        ":op_hash": receipt.receipt.dht_op_hash,
+                    },
+                    |row| row.get(0),
+                )?;
+
+                // If we have enough receipts then set receipts to complete.
+                if receipt_count >= required_validation_count as usize {
+                    set_receipts_complete(txn, &receipt.receipt.dht_op_hash, true)?;
+                }
+
                 // Add to receipts db
                 validation_receipts::add_if_unique(txn, receipt)
             })
@@ -610,67 +705,88 @@ impl Cell {
         Ok(())
     }
 
-    #[instrument(skip(self, dht_arc, since, until))]
+    #[instrument(skip(self))]
     /// the network module is requesting a list of dht op hashes
-    async fn handle_fetch_op_hashes_for_constraints(
+    /// Get the [`DhtOpHash`]es and authored timestamps for a given time window.
+    pub(super) async fn handle_query_op_hashes(
         &self,
-        dht_arc: holochain_p2p::dht_arc::DhtArc,
-        since: Timestamp,
-        until: Timestamp,
-    ) -> CellResult<Vec<DhtOpHash>> {
-        // FIXME: Test this query.
-        let full = (dht_arc.coverage() - 1.0).abs() < f64::EPSILON;
-        let result = self
-            .env()
-            .async_reader(move |txn| {
-                let r = if full {
-                    txn.prepare_cached(holochain_sqlite::sql::sql_cell::FETCH_OP_HASHES_FULL)?
-                        .query_map(
-                            named_params! {
-                                ":from": since.to_sql_ms_lossy(),
-                                ":to": until.to_sql_ms_lossy(),
-                            },
-                            |row| row.get("hash"),
-                        )?
-                        .collect::<rusqlite::Result<Vec<_>>>()?
-                } else if let Some((start_loc, end_loc)) = dht_arc.primitive_range_grouped() {
-                    let sql = if start_loc <= end_loc {
-                        holochain_sqlite::sql::sql_cell::FETCH_OP_HASHES_CONTINUOUS
-                    } else {
-                        holochain_sqlite::sql::sql_cell::FETCH_OP_HASHES_WRAPPED
-                    };
-                    txn.prepare_cached(sql)?
-                        .query_map(
-                            named_params! {
-                                ":from": since.to_sql_ms_lossy(),
-                                ":to": until.to_sql_ms_lossy(),
-                                ":storage_start_loc": start_loc,
-                                ":storage_end_loc": end_loc,
-                            },
-                            |row| row.get("hash"),
-                        )?
-                        .collect::<rusqlite::Result<Vec<_>>>()?
-                } else {
-                    Vec::new()
-                };
-                DatabaseResult::Ok(r)
-            })
-            .await?;
-        Ok(result)
+        dht_arc_set: DhtArcSet,
+        window_ms: TimeWindowMs,
+        include_limbo: bool,
+    ) -> CellResult<Vec<(DhtOpHash, u64)>> {
+        let mut results = Vec::new();
+
+        // The exclusive window bounds.
+        let start = clamp64(window_ms.start);
+        let end = clamp64(window_ms.end);
+
+        let full = if include_limbo {
+            holochain_sqlite::sql::sql_cell::any::FETCH_OP_HASHES_FULL
+        } else {
+            holochain_sqlite::sql::sql_cell::integrated::FETCH_OP_HASHES_FULL
+        };
+
+        let continuous = if include_limbo {
+            holochain_sqlite::sql::sql_cell::any::FETCH_OP_HASHES_CONTINUOUS
+        } else {
+            holochain_sqlite::sql::sql_cell::integrated::FETCH_OP_HASHES_CONTINUOUS
+        };
+
+        let wrapped = if include_limbo {
+            holochain_sqlite::sql::sql_cell::any::FETCH_OP_HASHES_WRAPPED
+        } else {
+            holochain_sqlite::sql::sql_cell::integrated::FETCH_OP_HASHES_WRAPPED
+        };
+
+        // For each interval in the set, fetch the hashes and timestamps.
+        for interval in dht_arc_set.intervals() {
+            let result = self
+                .env()
+                .async_reader(move |txn| {
+                    DatabaseResult::Ok(match interval {
+                        ArcInterval::Full => txn
+                            .prepare_cached(full)?
+                            .query_map(
+                                named_params! {
+                                    ":from": start,
+                                    ":to": end,
+                                },
+                                |row| Ok((row.get("hash")?, row.get("authored_timestamp_ms")?)),
+                            )?
+                            .collect::<rusqlite::Result<Vec<_>>>()?,
+                        ArcInterval::Bounded(start_loc, end_loc) => {
+                            let sql = if start_loc <= end_loc {
+                                continuous
+                            } else {
+                                wrapped
+                            };
+                            txn.prepare_cached(sql)?
+                                .query_map(
+                                    named_params! {
+                                        ":from": start,
+                                        ":to": end,
+                                        ":storage_start_loc": start_loc,
+                                        ":storage_end_loc": end_loc,
+                                    },
+                                    |row| Ok((row.get("hash")?, row.get("authored_timestamp_ms")?)),
+                                )?
+                                .collect::<rusqlite::Result<Vec<_>>>()?
+                        }
+                        ArcInterval::Empty => Vec::new(),
+                    })
+                })
+                .await?;
+            results.extend(result);
+        }
+        Ok(results)
     }
 
     #[instrument(skip(self, op_hashes))]
     /// The network module is requesting the content for dht ops
-    async fn handle_fetch_op_hash_data(
+    async fn handle_fetch_op_data(
         &self,
         op_hashes: Vec<holo_hash::DhtOpHash>,
-    ) -> CellResult<
-        Vec<(
-            holo_hash::AnyDhtHash,
-            holo_hash::DhtOpHash,
-            holochain_types::dht_op::DhtOp,
-        )>,
-    > {
+    ) -> CellResult<Vec<(holo_hash::DhtOpHash, holochain_types::dht_op::DhtOp)>> {
         // FIXME: Test this query.
         // TODO: SQL_PERF: Really on the fence about this query.
         // It has the potential to be slow if data density is very high
@@ -683,7 +799,7 @@ impl Cell {
                 positions.pop();
                 let sql = format!(
                     "
-                SELECT DhtOp.hash, DhtOp.basis_hash, DhtOp.type AS dht_type,
+                SELECT DhtOp.hash, DhtOp.type AS dht_type,
                 Header.blob AS header_blob, Entry.blob AS entry_blob
                 FROM DHtOp
                 JOIN Header ON DhtOp.header_hash = Header.hash
@@ -698,7 +814,6 @@ impl Cell {
                 let mut stmt = txn.prepare(&sql)?;
                 let r = stmt
                     .query_and_then(rusqlite::params_from_iter(op_hashes.into_iter()), |row| {
-                        let basis_hash: AnyDhtHash = row.get("basis_hash")?;
                         let header = from_blob::<SignedHeader>(row.get("header_blob")?)?;
                         let op_type: DhtOpType = row.get("dht_type")?;
                         let hash: DhtOpHash = row.get("hash")?;
@@ -717,7 +832,7 @@ impl Cell {
                             };
                         }
                         let op = DhtOp::from_type(op_type, header, entry)?;
-                        StateQueryResult::Ok((basis_hash, hash, op))
+                        StateQueryResult::Ok((hash, op))
                     })?
                     .collect::<StateQueryResult<Vec<_>>>()?;
                 StateQueryResult::Ok(r)
@@ -801,6 +916,7 @@ impl Cell {
             keystore,
             args,
             self.queue_triggers.publish_dht_ops.clone(),
+            self.queue_triggers.integrate_dht_ops.clone(),
         )
         .await
         .map_err(Box::new)?)
