@@ -1,5 +1,5 @@
 use crate::core::ribosome::FnComponents;
-use crate::core::ribosome::HostAccess;
+use crate::core::ribosome::HostContext;
 use crate::core::ribosome::Invocation;
 use crate::core::ribosome::ZomesToInvoke;
 use derive_more::Constructor;
@@ -28,7 +28,7 @@ pub struct PostCommitHostAccess {
     pub network: HolochainP2pCell,
 }
 
-impl From<PostCommitHostAccess> for HostAccess {
+impl From<PostCommitHostAccess> for HostContext {
     fn from(post_commit_host_access: PostCommitHostAccess) -> Self {
         Self::PostCommit(post_commit_host_access)
     }
