@@ -1,14 +1,14 @@
 use crate::core::ribosome::CallContext;
+use crate::core::ribosome::HostFnAccess;
 use crate::core::ribosome::RibosomeT;
 use holochain_p2p::HolochainP2pCellT;
+use holochain_types::access::Permission;
 use holochain_wasmer_host::prelude::WasmError;
 use holochain_zome_types::signal::RemoteSignal;
 use holochain_zome_types::zome::FunctionName;
 use holochain_zome_types::zome::ZomeName;
 use std::sync::Arc;
 use tracing::Instrument;
-use crate::core::ribosome::HostFnAccess;
-use holochain_types::access::Permission;
 
 #[tracing::instrument(skip(_ribosome, call_context, input))]
 pub fn remote_signal(
@@ -16,19 +16,22 @@ pub fn remote_signal(
     call_context: Arc<CallContext>,
     input: RemoteSignal,
 ) -> Result<(), WasmError> {
-    match HostFnAccess::from(&call_context.host_access()) {
-        HostFnAccess{ write_network: Permission::Allow, .. } => {
+    match HostFnAccess::from(&call_context.host_context()) {
+        HostFnAccess {
+            write_network: Permission::Allow,
+            ..
+        } => {
             const FN_NAME: &str = "recv_remote_signal";
             // Timeouts and errors are ignored,
             // this is a send and forget operation.
-            let network = call_context.host_access().network().clone();
+            let network = call_context.host_context().network().clone();
             let RemoteSignal { agents, signal } = input;
             let zome_name: ZomeName = call_context.zome().into();
             let fn_name: FunctionName = FN_NAME.into();
             for agent in agents {
                 tokio::task::spawn(
                     {
-                        let mut network = network.clone();
+                        let network = network.clone();
                         let zome_name = zome_name.clone();
                         let fn_name = fn_name.clone();
                         let payload = signal.clone();
@@ -51,7 +54,7 @@ pub fn remote_signal(
                 );
             }
             Ok(())
-        },
+        }
         _ => unreachable!(),
     }
 }
