@@ -5,6 +5,7 @@ use super::*;
 
 use crate::core::queue_consumer::TriggerSender;
 use crate::here;
+use crate::test_utils::test_network;
 use ::fixt::prelude::*;
 use holochain_sqlite::db::WriteManager;
 use holochain_state::query::link::GetLinksQuery;
@@ -166,8 +167,8 @@ impl Db {
                             .query_row(
                                 "
                                 SELECT EXISTS(
-                                    SELECT 1 FROM DhtOP 
-                                    WHERE when_integrated IS NOT NULL 
+                                    SELECT 1 FROM DhtOP
+                                    WHERE when_integrated IS NOT NULL
                                     AND hash = :hash
                                     AND validation_status = :status
                                 )
@@ -188,8 +189,8 @@ impl Db {
                             .query_row(
                                 "
                                 SELECT EXISTS(
-                                    SELECT 1 FROM DhtOP 
-                                    WHERE when_integrated IS NULL 
+                                    SELECT 1 FROM DhtOP
+                                    WHERE when_integrated IS NULL
                                     AND validation_stage = 3
                                     AND hash = :hash
                                     AND validation_status = :status
@@ -210,8 +211,8 @@ impl Db {
                             .query_row(
                                 "
                                 SELECT EXISTS(
-                                    SELECT 1 FROM DhtOP 
-                                    WHERE when_integrated IS NOT NULL 
+                                    SELECT 1 FROM DhtOP
+                                    WHERE when_integrated IS NOT NULL
                                     AND header_hash = :hash
                                     AND validation_status = :status
                                     AND (type = :store_entry OR type = :store_element)
@@ -237,7 +238,7 @@ impl Db {
                                 SELECT EXISTS(
                                     SELECT 1 FROM DhtOp
                                     JOIN Header ON DhtOp.header_hash = Header.hash
-                                    WHERE DhtOp.when_integrated IS NOT NULL 
+                                    WHERE DhtOp.when_integrated IS NOT NULL
                                     AND DhtOp.validation_status = :status
                                     AND (
                                         (Header.entry_hash = :hash AND DhtOp.type = :store_element)
@@ -265,8 +266,8 @@ impl Db {
                             .query_row(
                                 "
                                 SELECT EXISTS(
-                                    SELECT 1 FROM DhtOP 
-                                    WHERE when_integrated IS NOT NULL 
+                                    SELECT 1 FROM DhtOP
+                                    WHERE when_integrated IS NOT NULL
                                     AND basis_hash = :basis
                                     AND header_hash = :hash
                                     AND validation_status = :status
@@ -291,8 +292,8 @@ impl Db {
                             .query_row(
                                 "
                                 SELECT EXISTS(
-                                    SELECT 1 FROM DhtOP 
-                                    WHERE when_integrated IS NOT NULL 
+                                    SELECT 1 FROM DhtOP
+                                    WHERE when_integrated IS NOT NULL
                                     AND basis_hash = :basis
                                     AND header_hash = :hash
                                     AND validation_status = :status
@@ -316,8 +317,8 @@ impl Db {
                             .query_row(
                                 "
                                 SELECT EXISTS(
-                                    SELECT 1 FROM DhtOP 
-                                    WHERE when_integrated IS NOT NULL 
+                                    SELECT 1 FROM DhtOP
+                                    WHERE when_integrated IS NOT NULL
                                     AND basis_hash = :basis
                                     AND header_hash = :hash
                                     AND validation_status = :status
@@ -344,11 +345,11 @@ impl Db {
                                 SELECT EXISTS(
                                     SELECT 1 FROM DhtOP
                                     JOIN Header on DhtOp.header_hash = Header.hash
-                                    WHERE when_integrated IS NOT NULL 
+                                    WHERE when_integrated IS NOT NULL
                                     AND validation_status = :status
                                     AND (
                                         (DhtOp.type = :deleted_entry_header AND Header.deletes_header_hash = :deleted_header_hash)
-                                        OR 
+                                        OR
                                         (DhtOp.type = :deleted_by AND header_hash = :hash)
                                     )
                                 )
@@ -466,8 +467,9 @@ impl Db {
 
 async fn call_workflow<'env>(env: EnvWrite) {
     let (qt, _rx) = TriggerSender::new();
-    let (qt2, _rx) = TriggerSender::new();
-    integrate_dht_ops_workflow(env.clone(), qt, qt2)
+    let test_network = test_network(None, None).await;
+    let holochain_p2p_cell = test_network.cell_network();
+    integrate_dht_ops_workflow(env.clone(), qt, holochain_p2p_cell)
         .await
         .unwrap();
 }
@@ -800,7 +802,7 @@ async fn get_entry(env: EnvWrite, entry_hash: EntryHash) -> Option<Entry> {
         call_context.host_context = host_access.into();
         let ribosome = Arc::new(ribosome);
         let call_context = Arc::new(call_context);
-        host_fn::get::get(ribosome.clone(), call_context.clone(), input).unwrap()
+        host_fn::get::get(ribosome.clone(), call_context.clone(), vec![input]).unwrap()
     };
     output.and_then(|el| el.into())
 }
