@@ -5,6 +5,7 @@ use super::*;
 use crate::conductor::api::CellConductorApiT;
 use crate::core::queue_consumer::TriggerSender;
 use crate::core::queue_consumer::WorkComplete;
+use crate::core::sys_validate::check_and_hold_store_element;
 use crate::core::sys_validate::*;
 use crate::core::validation::*;
 use error::WorkflowResult;
@@ -12,9 +13,8 @@ use holo_hash::DhtOpHash;
 use holochain_cascade::Cascade;
 use holochain_p2p::HolochainP2pCell;
 use holochain_p2p::HolochainP2pCellT;
-use holochain_sqlite::prelude::*;
-
 use holochain_sqlite::db::ReadManager;
+use holochain_sqlite::prelude::*;
 use holochain_state::host_fn_workspace::HostFnStores;
 use holochain_state::host_fn_workspace::HostFnWorkspace;
 use holochain_state::prelude::*;
@@ -24,7 +24,6 @@ use holochain_zome_types::Entry;
 use holochain_zome_types::ValidationStatus;
 use std::convert::TryInto;
 use tracing::*;
-
 use types::Outcome;
 
 pub mod types;
@@ -192,6 +191,7 @@ fn handle_failed(error: ValidationOutcome) -> Outcome {
         ValidationOutcome::Counterfeit(_, _) => {
             unreachable!("Counterfeit ops are dropped before sys validation")
         }
+        ValidationOutcome::HeaderNotInCounterSigningSession(_, _) => Rejected,
         ValidationOutcome::DepMissingFromDht(_) => MissingDhtDep,
         ValidationOutcome::EntryDefId(_) => Rejected,
         ValidationOutcome::EntryHash => Rejected,
@@ -207,9 +207,11 @@ fn handle_failed(error: ValidationOutcome) -> Outcome {
         }
         ValidationOutcome::PrevHeaderError(_) => Rejected,
         ValidationOutcome::PrivateEntry => Rejected,
+        ValidationOutcome::PreflightResponseSignature(_) => Rejected,
         ValidationOutcome::UpdateTypeMismatch(_, _) => Rejected,
         ValidationOutcome::VerifySignature(_, _) => Rejected,
         ValidationOutcome::ZomeId(_) => Rejected,
+        ValidationOutcome::CounterSigningError(_) => Rejected,
     }
 }
 
@@ -224,6 +226,23 @@ async fn validate_op_inner(
         DhtOp::StoreElement(_, header, entry) => {
             store_element(header, workspace, network.clone()).await?;
             if let Some(entry) = entry {
+                // Retrieve for all other headers on countersigned entry.
+                if let Entry::CounterSign(session_data, _) = &**entry {
+                    let entry_hash = EntryHash::with_data_sync(&**entry);
+                    for header in session_data.build_header_set(entry_hash)? {
+                        let hh = HeaderHash::with_data_sync(&header);
+                        if workspace
+                            .full_cascade(network.clone())
+                            .retrieve_header(hh.clone(), Default::default())
+                            .await?
+                            .is_none()
+                        {
+                            return Err(SysValidationError::ValidationOutcome(
+                                ValidationOutcome::DepMissingFromDht(hh.into()),
+                            ));
+                        }
+                    }
+                }
                 store_entry(
                     (header)
                         .try_into()
@@ -238,6 +257,22 @@ async fn validate_op_inner(
             Ok(())
         }
         DhtOp::StoreEntry(_, header, entry) => {
+            // Check and hold for all other headers on countersigned entry.
+            if let Entry::CounterSign(session_data, _) = &**entry {
+                let dependency_check = |_original_element: &Element| Ok(());
+                let entry_hash = EntryHash::with_data_sync(&**entry);
+                for header in session_data.build_header_set(entry_hash)? {
+                    check_and_hold_store_element(
+                        &HeaderHash::with_data_sync(&header),
+                        workspace,
+                        network.clone(),
+                        incoming_dht_ops_sender.clone(),
+                        dependency_check,
+                    )
+                    .await?;
+                }
+            }
+
             store_entry(
                 (header).into(),
                 entry.as_ref(),
@@ -355,53 +390,77 @@ async fn sys_validate_element_inner(
 ) -> SysValidationResult<()> {
     let signature = element.signature();
     let header = element.header();
-    let entry = element.entry().as_option();
-    let incoming_dht_ops_sender = None;
-    if !counterfeit_check(signature, header).await? {
-        return Err(ValidationOutcome::Counterfeit(signature.clone(), header.clone()).into());
-    }
-    store_element(header, workspace, network.clone()).await?;
-    if let Some((entry, EntryVisibility::Public)) =
-        &entry.and_then(|e| header.entry_type().map(|et| (e, et.visibility())))
-    {
-        store_entry(
-            (header)
-                .try_into()
-                .map_err(|_| ValidationOutcome::NotNewEntry(header.clone()))?,
-            entry,
-            conductor_api,
-            workspace,
-            network.clone(),
-        )
-        .await?;
-    }
-    match header {
-        Header::Update(header) => {
-            register_updated_content(header, workspace, network, incoming_dht_ops_sender).await?;
+    let maybe_entry = element.entry().as_option();
+    counterfeit_check(signature, header).await?;
+
+    async fn validate(
+        header: &Header,
+        maybe_entry: Option<&Entry>,
+        workspace: &mut SysValidationWorkspace,
+        network: HolochainP2pCell,
+        conductor_api: &impl CellConductorApiT,
+    ) -> SysValidationResult<()> {
+        let incoming_dht_ops_sender = None;
+        store_element(header, workspace, network.clone()).await?;
+        if let Some((maybe_entry, EntryVisibility::Public)) =
+            &maybe_entry.and_then(|e| header.entry_type().map(|et| (e, et.visibility())))
+        {
+            store_entry(
+                (header)
+                    .try_into()
+                    .map_err(|_| ValidationOutcome::NotNewEntry(header.clone()))?,
+                maybe_entry,
+                conductor_api,
+                workspace,
+                network.clone(),
+            )
+            .await?;
         }
-        Header::Delete(header) => {
-            register_deleted_entry_header(header, workspace, network, incoming_dht_ops_sender)
+        match header {
+            Header::Update(header) => {
+                register_updated_content(header, workspace, network, incoming_dht_ops_sender)
+                    .await?;
+            }
+            Header::Delete(header) => {
+                register_deleted_entry_header(header, workspace, network, incoming_dht_ops_sender)
+                    .await?;
+            }
+            Header::CreateLink(header) => {
+                register_add_link(header, workspace, network, incoming_dht_ops_sender).await?;
+            }
+            Header::DeleteLink(header) => {
+                register_delete_link(header, workspace, network, incoming_dht_ops_sender).await?;
+            }
+            _ => {}
+        }
+        Ok(())
+    }
+
+    match maybe_entry {
+        Some(Entry::CounterSign(session, _)) => {
+            let entry_hash = EntryHash::with_data_sync(maybe_entry.unwrap());
+            for header in session.build_header_set(entry_hash)? {
+                validate(
+                    &header,
+                    maybe_entry,
+                    workspace,
+                    network.clone(),
+                    conductor_api,
+                )
                 .await?;
+            }
+            Ok(())
         }
-        Header::CreateLink(header) => {
-            register_add_link(header, workspace, network, incoming_dht_ops_sender).await?;
-        }
-        Header::DeleteLink(header) => {
-            register_delete_link(header, workspace, network, incoming_dht_ops_sender).await?;
-        }
-        _ => {}
+        _ => validate(header, maybe_entry, workspace, network, conductor_api).await,
     }
-    Ok(())
 }
 
 /// Check if the op has valid signature and author.
 /// Ops that fail this check should be dropped.
-pub async fn counterfeit_check(
-    signature: &Signature,
-    header: &Header,
-) -> SysValidationResult<bool> {
-    Ok(verify_header_signature(&signature, &header).await?
-        && author_key_is_valid(header.author()).await?)
+pub async fn counterfeit_check(signature: &Signature, header: &Header) -> SysValidationResult<()> {
+    verify_header_signature(&signature, &header).await?;
+    author_key_is_valid(header.author()).await?;
+    Ok(())
 }
 
 async fn register_agent_activity(
@@ -469,6 +528,7 @@ async fn store_entry(
         let entry_def = check_app_entry_type(app_entry_type, conductor_api).await?;
         check_not_private(&entry_def)?;
     }
+
     check_entry_hash(entry_hash, entry).await?;
     check_entry_size(entry)?;
 
@@ -484,6 +544,12 @@ async fn store_entry(
             })?;
         update_check(entry_update, original_header.header())?;
     }
+
+    // Additional checks if this is a countersigned entry.
+    if let Entry::CounterSign(session_data, _) = entry {
+        check_countersigning_session_data(EntryHash::with_data_sync(entry), session_data, header)
+            .await?;
+    }
     Ok(())
 }
 
