@@ -65,11 +65,11 @@ fn simple_crud_zome() -> InlineZome {
             Ok(hash)
         })
         .callback("read", |api, hash: HeaderHash| {
-            api.get(GetInput::new(hash.into(), GetOptions::default()))
+            api.get(vec![GetInput::new(hash.into(), GetOptions::default())])
                 .map_err(Into::into)
         })
         .callback("read_entry", |api, hash: EntryHash| {
-            api.get(GetInput::new(hash.into(), GetOptions::default()))
+            api.get(vec![GetInput::new(hash.into(), GetOptions::default())])
                 .map_err(Into::into)
         })
         // TODO: let this accept a usize, once the hdk refactor is merged
@@ -113,8 +113,12 @@ async fn inline_zome_2_agents_1_dna() -> anyhow::Result<()> {
     .await;
 
     // Verify that bobbo can run "read" on his cell and get alice's Header
-    let element: Option<Element> = conductor.call(&bobbo.zome("zome1"), "read", hash).await;
-    let element = element.expect("Element was None: bobbo couldn't `get` it");
+    let elements: Vec<Option<Element>> = conductor.call(&bobbo.zome("zome1"), "read", hash).await;
+    let element = elements
+        .into_iter()
+        .next()
+        .unwrap()
+        .expect("Element was None: bobbo couldn't `get` it");
 
     // Assert that the Element bobbo sees matches what alice committed
     assert_eq!(element.header().author(), alice.agent_pubkey());
@@ -174,10 +178,14 @@ async fn inline_zome_3_agents_2_dnas() -> anyhow::Result<()> {
 
     // Verify that bobbo can run "read" on his cell and get alice's Header
     // on the "foo" DNA
-    let element: Option<Element> = conductor
+    let elements: Vec<Option<Element>> = conductor
         .call(&bobbo_foo.zome("foozome"), "read", hash_foo)
         .await;
-    let element = element.expect("Element was None: bobbo couldn't `get` it");
+    let element = elements
+        .into_iter()
+        .next()
+        .unwrap()
+        .expect("Element was None: bobbo couldn't `get` it");
     assert_eq!(element.header().author(), alice_foo.agent_pubkey());
     assert_eq!(
         *element.entry(),
@@ -187,10 +195,14 @@ async fn inline_zome_3_agents_2_dnas() -> anyhow::Result<()> {
     // Verify that carol can run "read" on her cell and get alice's Header
     // on the "bar" DNA
     // Let's do it with the SweetZome instead of the SweetCell too, for fun
-    let element: Option<Element> = conductor
+    let elements: Vec<Option<Element>> = conductor
         .call(&carol_bar.zome("barzome"), "read", hash_bar)
         .await;
-    let element = element.expect("Element was None: carol couldn't `get` it");
+    let element = elements
+        .into_iter()
+        .next()
+        .unwrap()
+        .expect("Element was None: carol couldn't `get` it");
     assert_eq!(element.header().author(), alice_bar.agent_pubkey());
     assert_eq!(
         *element.entry(),
@@ -260,10 +272,14 @@ async fn get_deleted() -> anyhow::Result<()> {
 
     wait_for_integration_1m(alice.env(), expected_count).await;
 
-    let element: Option<Element> = conductor
+    let elements: Vec<Option<Element>> = conductor
         .call(&alice.zome("zome1"), "read", hash.clone())
         .await;
-    let element = element.expect("Element was None: bobbo couldn't `get` it");
+    let element = elements
+        .into_iter()
+        .next()
+        .unwrap()
+        .expect("Element was None: bobbo couldn't `get` it");
 
     assert_eq!(element.header().author(), alice.agent_pubkey());
     assert_eq!(
@@ -279,10 +295,10 @@ async fn get_deleted() -> anyhow::Result<()> {
     expected_count += WaitOps::DELETE;
     wait_for_integration_1m(alice.env(), expected_count).await;
 
-    let element: Option<Element> = conductor
+    let elements: Vec<Option<Element>> = conductor
         .call(&alice.zome("zome1"), "read_entry", entry_hash)
         .await;
-    assert!(element.is_none());
+    assert!(elements.into_iter().next().unwrap().is_none());
 
     Ok(())
 }
@@ -324,7 +340,7 @@ fn simple_validation_zome() -> InlineZome {
             Ok(hash)
         })
         .callback("read", |api, hash: HeaderHash| {
-            api.get(GetInput::new(hash.into(), GetOptions::default()))
+            api.get(vec![GetInput::new(hash.into(), GetOptions::default())])
                 .map_err(Into::into)
         })
         .callback("validate_create_entry", |_api, data: ValidateData| {
@@ -354,7 +370,8 @@ async fn simple_validation() -> anyhow::Result<()> {
 
     // This call passes validation
     let h1: HeaderHash = conductor.call(&alice, "create", AppString::new("A")).await;
-    let e1: Option<Element> = conductor.call(&alice, "read", &h1).await;
+    let e1s: Vec<Option<Element>> = conductor.call(&alice, "read", &h1).await;
+    let e1 = e1s.into_iter().next().unwrap();
     let s1: AppString = e1.unwrap().entry().to_app_option().unwrap().unwrap();
     assert_eq!(s1, AppString::new("A"));
 
